import doctest
import os
import re
import unittest
import zope.app.testing.functional


Layer = zope.app.testing.functional.ZCMLLayer(
    os.path.join(os.path.dirname(__file__), 'ftesting.zcml'),
    __name__, 'gocept.form.layer', allow_teardown=True)


# Strip out u'' literals in doctests, adapted from
# <https://stackoverflow.com/a/56507895>.
class Py23OutputChecker(doctest.OutputChecker, object):
    RE = re.compile(r"(\W|^)[uU]([rR]?[\'\"])", re.UNICODE)

    def remove_u(self, want, got):
        return (re.sub(self.RE, r'\1\2', want),
                re.sub(self.RE, r'\1\2', got))

    def check_output(self, want, got, optionflags):
        want, got = self.remove_u(want, got)
        return super(Py23OutputChecker, self).check_output(
            want, got, optionflags)

    def output_difference(self, example, got, optionflags):
        example.want, got = self.remove_u(example.want, got)
        return super(Py23OutputChecker, self).output_difference(
            example, got, optionflags)


def FunctionalDocFileSuite(*paths, **kw):
    try:
        layer = kw['layer']
    except KeyError:
        layer = Layer
    else:
        del kw['layer']
    kw['package'] = doctest._normalize_module(kw.get('package'))
    kw['checker'] = Py23OutputChecker()
    test = zope.app.testing.functional.FunctionalDocFileSuite(
        *paths, **kw)
    test.layer = layer
    return test


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(FunctionalDocFileSuite(
        'base.txt',
        'confirm-action.txt',
        'destructive-action.txt',
        'grouped.txt',
        'multiple-constraints.txt'
    ))
    return suite



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt

import zope.schema.interfaces


def applySchemaData(context, schema, data, omit=(), set_defaults=True):
    _marker = object()
    omit = set(omit)
    for name in schema.names(all=True):
        if name in omit:
            continue
        field = schema[name]
        if not zope.schema.interfaces.IField.providedBy(field):
            continue
        value = data.get(name, _marker)
        if value is _marker and set_defaults:
            value = field.default
        if value is not _marker:
            setattr(context, name, value)



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt

"""Pagelet support for z3c.form forms."""

import z3c.form.form
import z3c.pagelet.browser
import z3c.pagelet.interfaces
import zope.interface
import z3c.template.interfaces
import zope.component


@zope.interface.implementer(z3c.pagelet.interfaces.IPageletForm)
class Form(z3c.pagelet.browser.BrowserPagelet, z3c.form.form.Form):
    """Pagelet support for z3c.form.form.Form"""

    update = z3c.form.form.Form.update


@zope.interface.implementer(z3c.pagelet.interfaces.IPageletAddForm)
class AddForm(z3c.pagelet.browser.BrowserPagelet, z3c.form.form.AddForm):
    """Pagelet support for z3c.form.form.AddForm"""

    update = z3c.form.form.AddForm.update


@zope.interface.implementer(z3c.pagelet.interfaces.IPageletEditForm)
class EditForm(z3c.pagelet.browser.BrowserPagelet, z3c.form.form.EditForm):
    """Pagelet support for z3c.form.form.EditForm"""

    update = z3c.form.form.EditForm.update



# -*- coding: latin-1 -*-
# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
"""Additional actions."""

from base64 import b64encode
import sys
import xml.sax.saxutils

import zope.i18n
import zope.i18nmessageid
import zope.formlib.form
import zope.formlib.namedtemplate


class DestructiveAction(zope.formlib.form.Action):
    pass


@zope.formlib.namedtemplate.implementation(DestructiveAction)
def render_destructive_action(self):
    button = zope.formlib.form.render_submit_button(self)()
    if button == '':
        return ''

    # Extend the button with a check box that is synchronized with the
    # 'active' status of the button.

    additional = """
        <script type="text/javascript">
        document.getElementById('%s').disabled = true;
        </script>

        <label>
            <input
            type="checkbox"
            id="%s"
            onChange="document.getElementById('%s').disabled = !this.checked"/>
        Unlock &quot;%s&quot; button</label>

    """ % (self.__name__, self.__name__ + '.unlock', self.__name__, self.label)
    # XXX: the label should be translated.

    return button + additional


class destructive_action(zope.formlib.form.action):
    """Decorator to create destructive actions in form views."""

    def __call__(self, success):
        action = DestructiveAction(self.label, success=success, **self.options)
        self.actions.append(action)
        return action


class ConfirmAction(zope.formlib.form.Action):

    def __init__(self, label, confirm_message, **options):
        super(self.__class__, self).__init__(label, **options)
        self.confirm_message = confirm_message


@zope.formlib.namedtemplate.implementation(ConfirmAction)
def render_confirm_action(self):
    button = zope.formlib.form.render_submit_button(self)()
    if button == '':
        return ''

    # Extend the button with the confirmation.

    message = self.confirm_message
    if isinstance(message, zope.i18nmessageid.Message):
        message = zope.i18n.translate(message, context=self.form.request)

    additional = """
        <script type="text/javascript">
            function confirm_%(func_name)s(){
                var confirmed = confirm(%(message)s);
                if (confirmed)
                    return true;
                return false;
            }
            document.getElementById("%(name)s").onclick = confirm_%(func_name)s;
        </script>

    """ % dict(  # noqa
        name=self.__name__,
        func_name=b64encode(
            self.__name__.encode('utf-8')).decode('ascii')[:-2],
        message=xml.sax.saxutils.quoteattr(message))

    return button + additional


class confirm(object):
    """Decorator to create confirm actions in form views."""

    def __init__(self, label, actions=None, confirm_message=None, **options):
        if confirm_message is None:
            raise ValueError("No confirm message given.")
        caller_locals = sys._getframe(1).f_locals
        if actions is None:
            actions = caller_locals.get('actions')
        if actions is None:
            actions = caller_locals['actions'] = zope.formlib.form.Actions()
        self.actions = actions

        self.label = label
        self.options = options
        self.confirm_message = confirm_message

    def __call__(self, success):
        action = ConfirmAction(self.label, self.confirm_message,
                               success=success, **self.options)
        self.actions.append(action)
        return action



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
"""Tools for schema field constraints."""


def _all(list):
    """Emulates Python 2.5's all() function."""
    return bool(len([x for x in list if x]))


def all(*constraints):
    return lambda value: _all([c(value) for c in constraints])



from setuptools import setup


setup(
    name = 'gocept.form',
    version='0.9.0',
    author = "Christian Zagrodnick",
    author_email = "cz@gocept.com",
    description = "Extensions for zope.formlib",
    long_description = '\n\n'.join(
        open(name).read()
        for name in ['README.txt', 'CHANGES.txt']),
    license = "ZPL 2.1",
    url='http://pypi.python.org/pypi/gocept.form',

    include_package_data = True,
    zip_safe = False,

    install_requires = [
        'zope.interface',
        'zope.component',
        'zope.contentprovider',
        'zope.viewlet',
    ],
    extras_require = dict(
        test=[
              'zope.testbrowser',
              'zope.app.testing',
              'zope.app.zcmlfiles',
              'zope.viewlet!=3.4.1',
               'z3c.pagelet',
             ],
        formlib=[
            'zope.browserpage',
            'zope.formlib',
        ],
        z3cform=['z3c.form',
                 'z3c.pagelet'
                ])
    )
